/* ----------------------------------------------------------------------------
  f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
  Authors:  Bradley M. Bell    bradbell at washington dot edu
  License:  GNU Public License Version 2 or higher
-----------------------------------------------------------------------------*/
/*
$begin xerror2cpp$$
$spell
	cmlib
	xerror
	cpp
	nerr
	const
$$

$section Implementation of Cmlib xerror Routine in Cpp$$

$index xerror, C++$$
$index error, xerror$$

$head Syntax$$ 
$codei%xerror2cpp_(%message%, %n%, %nerr%, %level%)%$$

$head Description$$
The f2cad version of $cref xerror.f$$ uses this routine for its implementation.
This is not completely faithful to the specifications for $code xerror$$.

$head message$$
The argument $icode message$$ has prototype
$icode%
	const char *message
%$$
It points to the beginning of the error message to be printed.

$head n$$
The argument $icode n$$ has prototype
$icode%
	const int *n
%$$
It is the number of characters in the error message.

$head nerr$$
The argument $icode nerr$$ has prototype
$icode%
	const int *nerr
%$$
It is a number that identifies the error.

$head level$$
The argument $icode level$$ has prototype
$icode%
	const int *level
%$$
It has the following meaning:
$table
-1 $cnext a warning,  message is printed (but not multiple times in a row)
$rnext
0 $cnext a warning,  message is printed
$rnext
1 $cnext a non-fatal error, error number and message are printed
$rnext
2 $cnext a fatal error, error number and message are printed and program aborts
$tend

$end
*/

# include <cstdio>
# include <cassert>
# include <cstdlib>

# include <f2cad/f2cad.hpp>

namespace f2cad {
	
	void xerror2cpp_(
		integer *message  , 
		integer *n        ,
		integer *nerr     ,
		integer *level    )
	{	static int previous_nerr = 0;
	
		assert( *nerr != 0 );
	
		// check for an error case
		if( *level > 0 )
			std::printf("\nError number = %d\n", (int) *nerr);
	
		// print the message
		if( *level > -1 || *nerr != previous_nerr )
		{	int i;
			for(i = 0; i < *n; i++)
				std::putchar( (int) message[i]);
		}
	
		// check for a fatal error
		if( *level == 2 )
			exit(1);
	
		previous_nerr = *nerr;
		return;
	}
}
