# -----------------------------------------------------------------------------
# f2cad: Convert Fortran to C++ AD Types: Copyright (C) 2005-10 Bradley M. Bell
# Authors:  Bradley M. Bell    bradbell at washington dot edu
# License:  GNU Public License Version 2 or higher
# -----------------------------------------------------------------------------
#
s%F2C_INCLUDE%F2CAD_INCLUDED%g
#
# define f2cad interface routines 
s%^# *define *F2CAD_INCLUDED%&\
\
# include <cassert>\
\
// BEGIN Define test_result, integer and doublereal\
\
typedef enum { test_none, test_pass, test_fail } test_result;\
\
typedef long int integer;\
\
# ifdef F2CAD_USE_ADOLC\
# include <adolc/adouble.h>\
# include <adolc/interfaces.h>\
# include <adolc/taping.h>\
typedef adouble doublereal;\
# endif\
\
# ifdef F2CAD_USE_CPPAD\
# include <cppad/cppad.hpp>\
typedef CppAD::AD<double> doublereal;\
# endif\
\
# ifdef F2CAD_USE_FADBAD\
# include <FADBAD++/fadiff.h>\
typedef fadbad::F<double> doublereal;\
# endif\
\
// END Define test_result, integer and doublereal\
%
#
# integer and doublereal get defined above
/^typedef long int integer;/d
/^typedef double doublereal;/d
#
# remove all use of the f2c single precision type
/^typedef float real;/d
/^typedef struct { real r, i; } complex;/d
/^	real r;/d
/^typedef real (\*R_fp)(\.\.\.);/d
/^typedef real (\*R_fp)();/d
#
# place interface definitions after declaration of doublereal and integer 
s%^#define TRUE_ (1)%\
namespace f2cad {\
	// routines defined in lib/*/f2c_*.cpp\
	extern void Independent (integer n, doublereal *x);\
	extern void Dependent   (integer m, doublereal *y);\
	extern bool near_equal(double x, double y, double r, double a);\
	extern double Partial(const doublereal \&zero, integer i, integer j);\
	template <typename Type>\
	double Partial(integer i, integer j)\
	{	// template link to extra argument version\
		return Partial(doublereal(0), i, j);\
	}\
 	extern double Value(const doublereal \&z);\
}\
// necessary C routines that are part of f2c library\
extern "C" {\
	integer pow_ii(integer *, integer *);\
}\
// necessary C++ routines that replace part of f2c library\
# include <f2cad/dint.hpp>\
\
&%
#
# remove Multitype defintiion
s%^union Multitype {%# if 0\
&%
s%^typedef union Multitype Multitype;%&\
# endif%
#
# change dabs to abs (understood by all AD types)
s/^#define dabs(x) (doublereal)abs(x)/#define dabs(x) abs(x)/
#
# check if min is already defined (defined in adolc/common.h)
s%^#define min(a,b) ((a) <= (b) ? (a) : (b))%# ifndef min \
& \
# endif%
#
# check if max is already defined
s%^#define max(a,b) ((a) >= (b) ? (a) : (b))%# ifndef max \
& \
# endif%
